################################################################################
######## Estimate the genotype correlation matrix from reference panel #########
################################################################################

#' Code additively the genotypes
#' 
#' Sum the two columns per individual of the input matrix (each corresponding 
#' to one allele) to get the additive coding of the individual at the locus.
#'
#' @param genoMat is the matrix of genotype with two colums per individual
#' Rows are variants and columns are individuals
#' @param lind is the list of individual in the genotype matrix
#'
#' @return The additively coded genotype matrix
#'
getAdditivelyCodedMatrix <- function(genoMat, lind) {
    res <- matrix(0, nrow = nrow(genoMat), ncol = length(lind))
    rownames(res) <- rownames(genoMat)
    for (i in seq_along(lind)) {
        res[, i] <- rowSums(genoMat[, colnames(genoMat) %in% lind[[i]]])
    }
    res
}

################################################################################

#' Align the coding with the reference panel
#' 
#' Change the coding of the variant if the reference allele differs between 
#' the data and the reference panel. If reference alleles differs, 
#' switch 0s and 2s. Otherwise, no change.
#'
#' @param x is the matrix of additively coded genotypes
#' Rows are variants and columns are individuals
#' @param v is a vector of boolean indicating wether reference alleles differ or not
#'
#' @return The additively coded genotyped matrix with coded allele in the matrix
#' corresponding to the reference allele in the reference panel
#'
changeCoding <- function(x, v) {
    x[!v, ] <- 2 - x[!v, ]
    x
}

################################################################################

#' Compute the genotype correlation matrix.
#'
#' From a set of variants identified by the pair (chromosome, position), extract 
#' the SNPs from a reference panel in the specified population.
#'
#' There are two ways of retrieving data from reference panel:
#' 
#' * Through web queries. Currently, this is hard-coded to access 1000 Genomes phase3 data hosted by
#' Brian Browning (author of BEAGLE):
#'
#' \url{http://bochet.gcc.biostat.washington.edu/beagle/1000_Genomes_phase3_v5a/}
#'
#' * Locally, if one has locally downloaded the reference panel files from the URL above
#'  
#' This implementation discards multi-allelic markers that have a "," in the
#' ALT column.
#'
#' Position must be given in GRCh37 genome build.
#'
#' The \code{pop} can be any of: ACB, ASW, BEB, CDX, CEU, CHB, CHS, CLM, ESN,
#' FIN, GBR, GIH, GWD, IBS, ITU, JPT, KHV, LWK, MSL, MXL, PEL, PJL, PUR, STU,
#' TSI, YRI. It can also be any super-population: AFR, AMR, EAS, EUR, SAS.
#'
#' Then, code additively the genotype and modify the additively coded allele if 
#' reference alleles differ between data and reference panel
#' and finally compute the correlation matrix. Note that, by default, one SNP
#' in each pair with a correlation coefficient equal to 1 is removed.
#'
#' @param lrsid is a vector with the rs identifiers of the variants to extract
#' @param lchr is a vector with the chromosome number of the variants to extract
#' @param lpos is a vector with the physical position of the variants to extract
#' @param lrefall is a vector with the reference allele of the variant in the data.
#' @param pop is the 1000 Genomes code of the population in which data must be extracted
#' @param path is the path to the reference genotype file if locally downloaded (default is NULL)
#' @param web is a logical indicating whether to use web access or not (default is TRUE)
#' @param pruning is a logical indicating whether to prune the correlation matrix (default is TRUE)
#' @param rthresh is the threshold used to prune the correlation matrix (default is 1)
#'
#' @return The genotype correlation matrix of the specified variants
#'
#' @examples
#' 
#' ids = c("rs4841662", "rs34311866")
#' chrom <- c(8, 4)
#' phys_pos <- c(11843758, 951947)
#' refall <- c("A", "T")
#' cor_matrix <- getGenoCorMatrix(lrsid = ids, lchr = chrom, lpos = phys_pos, 
#'                                lrefall = refall, pop = "EUR")
#'
#' @export
#' 
getGenoCorMatrix <- function(lrsid, lchr, lpos, lrefall, pop, path = NULL, web = TRUE, pruning = TRUE, rthresh = 1) {
    if (length(lchr) > 1) {
        lpop <- rep(pop, length(lchr))
        referencedata <- mapply(get_vcf, lchr, lpos, lpos, lpop, MoreArgs = list(path, web), SIMPLIFY = FALSE)

        ind <- length(referencedata)
        
        # Discard SNPs not found in reference panel
        refs <- NULL
        nind <- NULL
        for (i in 1:ind) {
            if (!is.null(referencedata[[i]])) {
                refs <- cbind(refs, referencedata[[i]])
            }
            else {
                print(paste0("Warning: ", lrsid[i], " discarded (not found in reference panel)"))
                nind <- c(nind, i)
            }
        }
        
        if (!is.null(nind)) {
            lrsid <- lrsid[-nind]
            lrefall <- lrefall[-nind]
            ind <- ncol(refs)
        }
        
        if (length(lrsid) == 1) {
            return(1)
        }
        
        genoMat <- Reduce("rbind", lapply(1:dim(refs)[2], function(i) refs[, i]$geno))
        genoMap <- Reduce("rbind", lapply(1:dim(refs)[2], function(i) refs[, i]$meta))
        
        # Set allele to "t" when read allele is "TRUE"
        genoMap$REF <- vapply(genoMap$REF, function(x) 
            `if`(!(as.character(x) %in% c("A", "C", "G")), "T", x), "")
        
        # Keep only founders
        lind <- refs[, 1]$ind
        lind <- lind[lind$Paternal.ID == 0 & lind$Maternal.ID == 0, 2]
        
        testMat <- getAdditivelyCodedMatrix(genoMat,lind)
        to_remove <- NULL
        for (i in 1:nrow(testMat)) {
            if (!(rownames(testMat)[i] %in% lrsid)) {
                to_remove <- c(to_remove, i)
            }
        }
        if (!is.null(to_remove)) {
            testMat <- testMat[-to_remove, ]
            genoMap <- genoMap[-to_remove,] 
        }
        testMat <- changeCoding(testMat, as.character(lrefall) == genoMap$REF)
        colnames(testMat) <- lind
        cormat = stats::cor(t(testMat))
        if (pruning) {
            return(pruneCorMatrix(cormat, rthresh))
        }
        else {
            return(cormat)
        }
    }
    else {
        return(1)
    }
}

################################################################################

#' Remove SNPs in very high linkage disequilibrium
#' from the genotype correlation matrix.
#'
#' Parse a given a correlation matrix and discard one SNP in each pair of SNP
#' with a correlation coefficient greater than a specified threshold.
#'
#' @param matcor is a correlation matrix
#' @param rthresh is a threshold for the correlation coefficient
#' 
#' @return The genotype correlation matrix after removing one SNP per pair with
#' a correlation coefficient greater than \code{rthresh}

pruneCorMatrix <- function(matcor, rthresh) {
    to_rem <- unique(unlist(apply(matcor * upper.tri(matcor, diag = FALSE), 1, 
                           function(x) which(abs(x) >= rthresh))))
    if (length(to_rem) > 1) {
        matcor <- matcor[-to_rem, -to_rem]
        if (!is.null(nrow(matcor))) {
            return(matcor)
        }
        else {
            return(1)
        }
    }
    else {
        return(matcor)
    }
}
